import os
from azureml.core import Dataset
from .project import AzureProject
from .exceptions import AzureException
from .decorators import error_handler

class AzureDataset(object):

    def __init__(self, ctx):
        super(AzureDataset, self).__init__()
        self.ctx = ctx
        self.ws = None

    @error_handler
    def list(self):
        selected = self.ctx.config.get('dataset', None)
        datasets = Dataset.get_all(self._get_ws())
        ndatasts = len(datasets)
        for name in datasets.keys():
            self.ctx.log(
                ('[%s] ' % ('x' if selected == name else ' ')) + name)
        self.ctx.log('%s DataSet(s) listed' % ndatasts)
        return {'datasets': [name for name in datasets.keys()]}

    @error_handler
    def create(self, source = None, validation=False):
        ws = self._get_ws(True)
        if source is None:
            source = self.ctx.config.get('source', None)
        if source is None:
            raise AzureException('Please specify data source file...')

        if source.startswith("http:") or source.startswith("https:"):
            dataset = Dataset.Tabular.from_delimited_files(path=source)
            dataset_name = os.path.basename(source)
        else:    
            ds = self.ws.get_default_datastore()

            if self.ctx.config.path and not source.startswith("/"):
                source = os.path.join(self.ctx.config.path, source)

            ds.upload_files(files=[source], relative_root=None,
                target_path=None, overwrite=True, show_progress=True)
            dataset_name = os.path.basename(source)
            dataset = Dataset.Tabular.from_delimited_files(
                path=ds.path(dataset_name))

        dataset.register(workspace = ws, name = dataset_name,
            create_new_version = True)
        self._select(dataset_name, validation)
        self.ctx.log('Created DataSet %s' % dataset_name)
        return {'dataset': dataset_name}

    @error_handler
    def delete(self, name = None):
        ws = self._get_ws()
        if name is None:
            name = self.ctx.config.get('dataset', None)
        if name is None:
            raise AzureException('Please specify dataset name...')
        ds = Dataset.get_by_name(ws, name)
        ds.unregister_all_versions()
        self._select(None, False)
        self.ctx.log('Deleted dataset %s' % name)
        return {'deleted': name}

    @error_handler        
    def select(self, name = None):
        self._select(name)
        self.ctx.log('Selected dataset %s' % name)
        return {'selected': name}

    def _select(self, name, validation):
        if validation:
            self.ctx.config.set('azure', 'validation_dataset', name)
        else:
            self.ctx.config.set('azure', 'dataset', name)

        self.ctx.config.write('azure')

    def _get_ws(self, create_if_not_exist = False):
        if self.ws is None:
            self.ws = AzureProject(self.ctx)._get_ws(
                create_if_not_exist=create_if_not_exist)
        return self.ws



from a2ml.api.base_a2ml import BaseA2ML
from a2ml.api.utils.show_result import show_result

class A2MLDataset(BaseA2ML):
    """Contains the dataset CRUD operations that interact with provider."""
    def __init__(self, ctx, provider):
        """Initializes a new a2ml dataset.

        Args:
            context (object): An instance of the a2ml Context.
            provider (str): The automl provider/s you wish to run. For example 'auger,azure,google'.
        
        Returns:
            A2MLDataset object

        Examples:
            .. code-block:: python

                ctx = Context()
                dataset = A2MLDataset(ctx, 'auger, azure')
        """
        super(A2MLDataset, self).__init__()
        self.ctx = ctx
        self.runner = self.build_runner(ctx, provider, 'dataset')

    @show_result
    def list(self):
        """List all of the DataSets for the Project specified in the .yaml.

        Note:
            You will need to user the `iter <https://www.programiz.com/python-programming/methods/built-in/iter>`_ function to access the dataset elements.
        
        Returns:
            Results for each provider. ::

                {
                    'auger': {
                        'result': True,
                        'data': {
                            'datasets': <object> 
                        }
                    }
                }

        Examples:
            .. code-block:: python

                ctx = Context()
                dataset_list = A2MLDataset(ctx, 'auger, azure').list()
                for provider in ['auger', 'azure']
                    if dataset_list[provider].result is True:
                        for dataset in iter(dataset_list[provider].data.datasets):
                            ctx.log(dataset.get('name'))
                    else:
                        ctx.log('error %s' % dataset_list[provider].data)
        """
        return self.runner.execute('list')

    @show_result
    def create(self, source = None):
        """Create a new DataSet for the Project specified in the .yaml.

        Args:
            source(str): path to local or link to remote file.

        Returns:
            Results for each provider. ::

                    {
                        'auger': {
                            'result': True,
                            'data': {
                                'created': 'dataset.csv' 
                            }
                        }
                    }

        Examples:
            .. code-block:: python

                ctx = Context()
                dataset = DataSet(ctx, 'auger, azure').create('../dataset.csv')
        """
        return self.runner.execute('create', source)

    @show_result
    def delete(self, name = None):
        """
        Deletes a DataSet for the Project specified in the .yaml.

        Args:
            name(str): name of dataset.

        Returns:
            Results for each provider. ::

                    {
                        'auger': {
                            'result': True,
                            'data': {
                                'deleted': 'dataset.csv' 
                            }
                        }
                    }

        Examples:
            .. code-block:: python

                ctx = Context()
                DataSet(ctx, 'auger, azure').delete(dataset_name)
                ctx.log('Deleted dataset %s' % dataset_name)
        """
        return self.runner.execute('delete', name)

    @show_result

    def select(self, name = None):
        """
        Sets a DataSet name in the context.

        Args:
            name(str): name of dataset.

        Returns:
            Results for each provider. ::

                {
                    'auger': {
                        'result': True,
                        'data': {
                            'selected': 'fortunetest'
                        }
                    }
                }

        Examples:
            .. code-block:: python

            ctx = Context()
            DataSet(ctx, 'auger, azure').select(dataset_name)
        """
        return self.runner.execute('select', name)




from .impl.dataset import DataSet
from .impl.decorators import error_handler, authenticated, with_project
from .impl.cloud.rest_api import RestApi
from .credentials import Credentials
from .config import AugerConfig


class AugerDataset(object):
    def __init__(self, ctx):
        self.ctx = ctx
        self.ctx.credentials = Credentials(ctx).load()
        self.ctx.rest_api = RestApi(
            self.ctx.credentials.api_url, self.ctx.credentials.token)

    @error_handler
    @authenticated
    @with_project(autocreate=False)
    def list(self, project):
        count = 0
        selected = self.ctx.config.get('dataset', None)
        for dataset in iter(DataSet(self.ctx, project).list()):
            self.ctx.log(
                ('[%s] ' % ('x' if selected == dataset.get('name') else ' ')) +
                dataset.get('name')
            )
            count += 1
        self.ctx.log('%s DataSet(s) listed' % str(count))
        return {'datasets': DataSet(self.ctx, project).list()}

    @error_handler
    @authenticated
    @with_project(autocreate=True)
    def create(self, project, source = None, validation=False):
        if source is None:
            source = self.ctx.config.get('source', None)
        dataset = DataSet(self.ctx, project).create(source)
        AugerConfig(self.ctx).set_data_set(dataset.name, source, validation)
        self.ctx.log('Created DataSet %s' % dataset.name)
        return {'created': dataset.name}

    @error_handler
    @authenticated
    @with_project(autocreate=False)
    def delete(self, project, name):
        if name is None:
            name = self.ctx.config.get('dataset', None)
        DataSet(self.ctx, project, name).delete()
        if name == self.ctx.config.get('dataset', None):
            AugerConfig(self.ctx).set_data_set(None, None, False).set_experiment(None, None)
        self.ctx.log('Deleted dataset %s' % name)
        return {'deleted': name}

    @error_handler
    def select(self, name):
        old_name = self.ctx.config.get('dataset', None)
        if name != old_name:
            AugerConfig(self.ctx).set_data_set(name, None, False).set_experiment(None, None)
        self.ctx.log('Selected DataSet %s' % name)
        return {'selected': name}

    @error_handler
    @authenticated
    @with_project(autocreate=False)
    def download(self, project, name, path_to_download):
        if name is None:
            name = self.ctx.config.get('dataset', None)
        file_name = DataSet(self.ctx, project, name).download(path_to_download)
        self.ctx.log('Downloaded dataset %s to %s' % (name, file_name))
        return {'dowloaded': name, 'file': file_name}



import os
import logging
import pandas as pd
import datetime as dt
from azureml.core import Run
from azureml.core import Dataset
from azureml.core import Experiment
from azureml.core.compute import AmlCompute
from azureml.core.compute import ComputeTarget
from azureml.train.automl import AutoMLConfig
from azureml.train.automl.run import AutoMLRun
from azureml.automl.core.featurization import FeaturizationConfig
from .project import AzureProject
from .exceptions import AzureException
from .decorators import error_handler
from a2ml.api.utils.formatter import print_table
from a2ml.api.azure.dataset import AzureDataset

class AzureExperiment(object):

    def __init__(self, ctx):
        super(AzureExperiment, self).__init__()
        self.ctx = ctx

    @error_handler    
    def list(self):
        ws = AzureProject(self.ctx)._get_ws()
        experiments = Experiment.list(workspace=ws)
        nexperiments = len(experiments)
        experiments = [e.name for e in experiments]
        for name in experiments:
            self.ctx.log(name)
        self.ctx.log('%s Experiment(s) listed' % str(nexperiments))
        return {'experiments': experiments}

    @error_handler    
    def start(self):
        ws = AzureProject(self.ctx)._get_ws()

        dataset_name = self.ctx.config.get('dataset', None)
        if dataset_name is None:
            raise AzureException('Please specify Dataset name...')
        experiment_name = self._fix_name(
            self.ctx.config.get('experiment/name', dataset_name))
        cluster_name = self._fix_name(
            self.ctx.config.get('cluster/name', 'cpucluster'))

        self.ctx.log("Starting search on %s Dataset..." % dataset_name)
        dataset = Dataset.get_by_name(ws, dataset_name)
        #TODO dataset = dataset.drop_columns(columns)

        compute_target = self._get_compute_target(ws, cluster_name)

        model_type = self.ctx.config.get('model_type')
        if not model_type:
            raise AzureException('Please specify model type...')
        primary_metric = self.ctx.config.get(
            'experiment/metric','spearman_correlation')
        if not primary_metric:
            raise AzureException('Please specify primary metric...')
        #TODO: check if primary_metric is constent with model_type
        target = self.ctx.config.get('target')
        if not target:
            raise AzureException('Please specify target column...')

        automl_settings = {
            "iteration_timeout_minutes" : self.ctx.config.get(
                'experiment/max_eval_time',10),
            "iterations" : self.ctx.config.get(
                'experiment/max_n_trials',10),
            "primary_metric" : primary_metric,
            "verbosity" : logging.INFO,
            "enable_stack_ensemble": self.ctx.config.get(
                'experiment/use_ensemble', False)
        }

        validation_data = None
        if self.ctx.config.get('experiment/validation_data'):
            if self.ctx.config.get('validation_dataset'):
                validation_data = Dataset.get_by_name(ws, self.ctx.config.get('validation_dataset'))
            if not validation_data:
                res = AzureDataset(self.ctx).create(
                    source = self.ctx.config.get('experiment/validation_data'),
                    validation = True
                )
                validation_data = Dataset.get_by_name(ws, res['dataset'])
        else:    
            automl_settings["n_cross_validations"] = self.ctx.config.get(
                'experiment/cross_validation_folds', 5)
            if self.ctx.config.get('experiment/validation_size'):
                automl_settings["validation_size"] = self.ctx.config.get('experiment/validation_size')

        if self.ctx.config.get('experiment/max_total_time'):
            automl_settings["experiment_timeout_hours"] = float(self.ctx.config.get('experiment/max_total_time'))/60.0

        if self.ctx.config.get('exclude'):
            fc = FeaturizationConfig()
            fc.drop_columns = self.ctx.config.get('exclude').split(",")
            automl_settings["featurization"] = fc

        automl_config = AutoMLConfig(
            task = model_type,
            debug_log = 'automl_errors.log',
            path = os.getcwd(),
            compute_target = compute_target,
            training_data = dataset,
            validation_data = validation_data,
            label_column_name = target,
            **automl_settings)

        experiment = Experiment(ws, experiment_name)
        run = experiment.submit(automl_config, show_output = False)
        self.ctx.log("Started Experiment %s search..." % experiment_name)
        self.ctx.config.set('azure', 'experiment/name', experiment_name)
        self.ctx.config.set('azure', 'experiment/run_id', run.run_id)
        self.ctx.config.write('azure')

        return {'experiment_name': experiment_name, 'run_id': run.run_id}

    @error_handler    
    def stop(self):
        ws = AzureProject(self.ctx)._get_ws()
        experiment_name = self.ctx.config.get('experiment/name', None)
        if experiment_name is None:
            raise AzureException('Please specify Experiment name...')
        run_id = self.ctx.config.get('experiment/run_id', None)
        if run_id is None:
            raise AzureException(
                'Pleae provide Run ID (experiment/run_id)...')
        experiment = Experiment(ws, experiment_name)
        run = AutoMLRun(experiment = experiment, run_id = run_id)
        run.cancel()
        return {'stopped': experiment_name}

    @error_handler    
    def leaderboard(self, run_id = None):
        ws = AzureProject(self.ctx)._get_ws()
        experiment_name = self.ctx.config.get('experiment/name', None)
        if experiment_name is None:
            raise AzureException('Please specify Experiment name...')
        if run_id is None:
            run_id = self.ctx.config.get('experiment/run_id', None)
        if run_id is None:
            raise AzureException(
                'Pleae provide Run ID (experiment/run_id) to evaluate')
        experiment = Experiment(ws, experiment_name)
        run = AutoMLRun(experiment = experiment, run_id = run_id)
        leaderboard = self._get_leaderboard(run).to_dict('records')
        self.ctx.log('Leaderboard for Run %s' % run_id)
        print_table(self.ctx.log,leaderboard)
        status = run.get_status()
        self.ctx.log('Status: %s' % status)
        return {
            'run_id': run_id,
            'leaderboard': leaderboard,
            'status': status }

    @error_handler        
    def get_experiment_settings(self):
        return 
                
    @error_handler        
    def history(self):
        ws = AzureProject(self.ctx)._get_ws()
        experiment_name = self.ctx.config.get('experiment/name', None)
        if experiment_name is None:
            raise AzureException('Please specify Experiment name...')
        experiment = Experiment(ws, experiment_name)
        runs = Run.list(experiment)
        result = []
        for run in runs:
            details = run.get_details()
            st = dt.datetime.strptime(
                details['startTimeUtc'], '%Y-%m-%dT%H:%M:%S.%fZ')
            et = dt.datetime.strptime(
                details['endTimeUtc'], '%Y-%m-%dT%H:%M:%S.%fZ')
            duratin = str(et-st)
            result.append({
                'id': run.id,
                'start time': details['startTimeUtc'],
                'duratin': duratin,
                'status': details['status']
            })
        print_table(self.ctx.log, result)
        return {'history': result}

    def _fix_name(self, name):
        # Name can include letters, digits and dashes.
        # It must start with a letter, end with a letter or digit,
        # and be between 2 and 16 characters in length.
        #TODO check for all conditions
        return name.replace('_','-').replace('.','-')

    def _get_compute_target(self, ws, cluster_name):
        compute_min_nodes = self.ctx.config.get('cluster/min_nodes',1)
        compute_max_nodes = self.ctx.config.get('cluster/max_nodes',4)
        compute_sku = self.ctx.config.get('cluster/type','STANDARD_D2_V2')

        if cluster_name in ws.compute_targets:
            compute_target = ws.compute_targets[cluster_name]
            if compute_target and type(compute_target) is AmlCompute:
                self.ctx.log(
                    'Found compute target %s ...' % cluster_name)
        else:
            self.ctx.log('Creating new AML compute context...')
            provisioning_config = AmlCompute.provisioning_configuration(
                vm_size=compute_sku, min_nodes=compute_min_nodes,
                max_nodes=compute_max_nodes)
            compute_target = ComputeTarget.create(
                ws, cluster_name, provisioning_config)
            compute_target.wait_for_completion(show_output = True)
        return compute_target

    def _get_leaderboard(self, run):
        primary_metric = run.properties['primary_metric']
        children = list(run.get_children(recursive=True))
        leaderboard = pd.DataFrame(index=['model id', 'algorithm', 'score'])
        goal_minimize = False
        for run in children:
            if('run_algorithm' in run.properties and 'score' in run.properties):
                if(run.properties['run_preprocessor']):
                    run_algorithm = '%s,%s' % (run.properties['run_preprocessor'],
                        run.properties['run_algorithm'])
                else:
                    run_algorithm = run.properties['run_algorithm']
                leaderboard[run.id] = [run.id,
                                      run_algorithm,
                                      float(run.properties['score'])]
                if('goal' in run.properties):
                    goal_minimize = run.properties['goal'].split('_')[-1] == 'min'

        leaderboard = leaderboard.T.sort_values(
            'score', ascending = goal_minimize)
        leaderboard = leaderboard.head(10)
        leaderboard.rename(columns={'score':primary_metric}, inplace=True)
        return leaderboard



import logging
import shutil
import os
import json

import pytest
from a2ml.api.utils.context import Context
from a2ml.api.auger.impl.cloud.rest_api import RestApi

# from click.testing import CliRunner

#from auger.api.credentials import Credentials
#from auger.api.cloud.rest_api import RestApi

TEST_CREDENTIALS = {
    'username': 'test_user',
    'organization': 'auger',
    'api_url': 'https://example.com',
    'token': 'fake_token',
}

# @pytest.fixture
# def runner():
#     return CliRunner()


# @pytest.fixture(scope="function")
# def isolated(runner):
#     with runner.isolated_filesystem():
#         yield runner


@pytest.fixture
def log(caplog):
    caplog.set_level(logging.INFO)
    return caplog


@pytest.fixture
def project(isolated):
    source = os.path.join(
        os.path.dirname(os.path.abspath(__file__)), '..', 'fixtures',
        'cli-integration-test')
    shutil.copytree(source, './test_project')
    os.chdir('test_project')


@pytest.fixture
def ctx(project):
    ctx = Context(debug = False)
    ctx.config.set('config', 'providers', ["auger"])
    ctx.config.set('config', 'experiment/name', 'iris-1.csv-experiment')
    ctx.config.set('config', 'dataset', 'iris.csv')

    return ctx

@pytest.fixture
def ctx_api():
    # load config(s) from the test app
    ctx = Context(debug = True)
    ctx.config.set('config', 'providers', ["auger"])
    ctx.rest_api = RestApi('api_url', 'token')
    return ctx

@pytest.fixture
def authenticated(monkeypatch, isolated):
    monkeypatch.setenv("AUGER_CREDENTIALS", json.dumps(TEST_CREDENTIALS))
    #monkeypatch.setenv("AUGER_CREDENTIALS_PATH", os.getcwd())


@pytest.fixture(autouse=True)
def no_requests(monkeypatch):
    def request(*args, **kwargs):
        print("CALLED HubApiClient.request(", args, kwargs, ")")
        raise Exception("No way further")
        return {}
    monkeypatch.setattr('auger.hub_api_client.HubApiClient.request', request)

# import os
# import shutil
# import logging
# from click.testing import CliRunner
# from a2ml.api.utils.context import Context
# from a2ml.api.auger.impl.rest_api import RestApi


# @pytest.fixture
# def ctx():
#     # load config(s) from the test app
#     ctx = Context()
#     ctx.rest_api = RestApi('api_url', 'token')
#     return ctx

# @pytest.fixture
# def runner():
#     return CliRunner()

# @pytest.fixture(scope="function")
# def isolated(runner):
#     with runner.isolated_filesystem():
#         yield runner

# @pytest.fixture
# def log(caplog):
#     caplog.set_level(logging.INFO)
#     return caplog

# @pytest.fixture
# def project(isolated):
#     source = os.path.join(
#         os.path.dirname(os.path.abspath(__file__)), '..', 'fixtures',
#         'cli-integration-test')
#     shutil.copytree(source, './cli-integration-test')
#     os.chdir('cli-integration-test')



import os
import json
from azureml.core import Experiment
from azureml.core.model import Model
from azureml.core.model import InferenceConfig
from azureml.core.webservice import Webservice
from azureml.core.webservice import AciWebservice
from azureml.exceptions import WebserviceException
from azureml.train.automl.run import AutoMLRun
from .project import AzureProject
from .exceptions import AzureException
from .decorators import error_handler
from a2ml.api.utils.dataframe import DataFrame


class AzureModel(object):

    def __init__(self, ctx):
        super(AzureModel, self).__init__()
        self.ctx = ctx

    @error_handler
    def deploy(self, model_id, locally):
        if locally:
            self.ctx.log('Local deployment step is not required for Azure..')
            return {'model_id': model_id}

        ws = AzureProject(self.ctx)._get_ws()
        experiment_name = self.ctx.config.get('experiment/name', None)
        if experiment_name is None:
            raise AzureException('Please specify Experiment name...')

        iteration, run_id = self._get_iteration(model_id)

        experiment = Experiment(ws, experiment_name)
        experiment_run = AutoMLRun(experiment = experiment, run_id = run_id)
        model_run = AutoMLRun(experiment = experiment, run_id = model_id)
        model_name = model_run.properties['model_name']
        self.ctx.log('Regestiring model: %s' % model_name)

        description = '%s-%s' % (model_name, iteration)
        model = experiment_run.register_model(
            model_name = model_name, iteration=iteration,
            description = description, tags = None)

        script_file_name = '.azureml/score_script.py'
        model_run.download_file(
            'outputs/scoring_file_v_1_0_0.py', script_file_name)

        # Deploying ACI Service
        aci_service_name = self._aci_service_name(model_name)
        self.ctx.log('Deploying AciWebservice %s ...' % aci_service_name)

        inference_config = InferenceConfig(
            environment = model_run.get_environment(),
            entry_script = script_file_name)

        aciconfig = AciWebservice.deploy_configuration(
            cpu_cores = 1,
            memory_gb = 2,
            tags = {'type': "inference-%s" % aci_service_name},
            description = "inference-%s" % aci_service_name)

        # Remove any existing service under the same name.
        try:
            Webservice(ws, aci_service_name).delete()
            self.ctx.log('Remove any existing service under the same name...')
        except WebserviceException:
            pass

        aci_service = Model.deploy(
            ws, aci_service_name, [model], inference_config, aciconfig)
        aci_service.wait_for_deployment(True)
        self.ctx.log('%s state %s' % (aci_service_name, str(aci_service.state)))

        return {'model_id': model_id, 'aci_service_name': aci_service_name}

    @error_handler
    def predict(self, filename, model_id, threshold, locally):
        ws = AzureProject(self.ctx)._get_ws()
        experiment_name = self.ctx.config.get('experiment/name', None)
        if experiment_name is None:
            raise AzureException('Please specify Experiment name...')
        experiment = Experiment(ws, experiment_name)

        target = self.ctx.config.get('target', None)
        predict_data = DataFrame.load(filename, target)

        y_pred = []
        if locally:
            y_pred, y_proba, proba_classes = self._predict_locally(
                experiment, predict_data, model_id, threshold)
        else:
            y_pred, y_proba, proba_classes = self._predict_remotely(
                ws, experiment, predict_data, model_id, threshold)

        predict_data[target] = y_pred

        if y_proba is not None:
            for idx, name in enumerate(proba_classes):
                predict_data['proba_'+str(name)] = list(y_proba[:,idx])

        predicted = self._save_predictions(predict_data, filename)

        return {'predicted': predicted}

    @error_handler        
    def actual(self, filename, model_id):
        pass

    def _get_iteration(self, model_id):
        iteration = None
        run_id = model_id
        parts = model_id.split('_')
        if len(parts) > 2:
            run_id = parts[0]+"_"+parts[1]
            iteration = parts[2]
        return iteration, run_id

    def _aci_service_name(self, model_name):
        # It must only consist of lowercase letters, numbers, or dashes, start
        # with a letter, end with a letter or number, and be between 3 and 32
        # characters long.
        #TODO - service_name + suffix must satisfy requiremets
        return (model_name+'-service').lower()

    def _predict_remotely(
        self, ws, experiment, predict_data, model_id, threshold):
        input_payload = predict_data.to_json(orient='split', index = False)

        remote_run = AutoMLRun(experiment = experiment, run_id = model_id)
        model_name = remote_run.properties['model_name']
        aci_service_name = self._aci_service_name(model_name)
        aci_service = AciWebservice(ws, aci_service_name)

        input_payload = json.loads(input_payload)
        # If you have a classification model, you can get probabilities by changing this to 'predict_proba'.        
        method = 'predict'
        if threshold is not None:
            method = 'predict_proba'
        input_payload = {
            'method': method,
            'data': input_payload['data']
        }
        input_payload = json.dumps(input_payload)
        try:
            response = aci_service.run(input_data = input_payload)
            print(response)
        except Exception as e:
            print('err log', aci_service.get_logs())
            raise e

        results_proba = None
        proba_classes = None

        return json.loads(response)['result'], results_proba, proba_classes

    def _predict_locally(self, experiment, predict_data, model_id, threshold):
        run_id = model_id
        iteration = None
        parts = model_id.split('_')
        if len(parts) > 2:
            run_id = parts[0]+"_"+parts[1]
            iteration = parts[2]

        remote_run = AutoMLRun(experiment = experiment, run_id = run_id)
        best_run, fitted_model = remote_run.get_output(iteration=iteration)

        results_proba = None
        proba_classes = None
        if threshold is not None:
            results_proba = fitted_model.predict_proba(predict_data)

            proba_classes = list(fitted_model.classes_)

            result = self._calculate_proba_target(results_proba,
                proba_classes, None, threshold, None)
        else:
            result = fitted_model.predict(predict_data)

        return result, results_proba, proba_classes

    def _calculate_proba_target(self, results_proba, proba_classes, proba_classes_orig, threshold, minority_target_class=None):
        import json
        results = []

        if type(threshold) == str:
            try:
                threshold = float(threshold)
            except:
                try:
                    threshold = json.loads(threshold)
                except Exception as e:
                    raise Exception("Threshold '%s' should be float or hash with target classes. Error: %s"%(threshold, str(e)))

        if type(threshold) != dict and minority_target_class is not None:
            threshold = {minority_target_class:threshold}

        print("Prediction threshold: %s, %s"%(threshold, proba_classes_orig))
        #print(results_proba)
        if type(threshold) == dict:
            mapped_threshold = {}
            if not proba_classes_orig:
                proba_classes_orig = proba_classes

            for name, value in threshold.items():
                idx_class = None
                for idx, item in enumerate(proba_classes_orig):
                    if item == name:
                        idx_class = idx
                        break

                if idx_class is None:
                    raise Exception("Unknown target class in threshold: %s, %s"%(name, proba_classes_orig))

                mapped_threshold[idx_class] = value

            for item in results_proba:
                proba_idx = None
                for idx, value in mapped_threshold.items():
                    if item[idx] >= value:
                        proba_idx = idx
                        break

                if proba_idx is None:
                    proba_idx = 0
                    for idx, value in enumerate(item):
                        if idx not in mapped_threshold:
                            proba_idx = idx
                            break

                results.append(proba_classes[proba_idx])
        else:
            #TODO: support multiclass classification
            for item in results_proba:
                max_proba_idx = 0
                for idx, prob in enumerate(item):
                    if prob > item[max_proba_idx]:
                        max_proba_idx = idx

                if item[max_proba_idx] < threshold:
                    if max_proba_idx > 0:
                        max_proba_idx = 0
                    else:
                        max_proba_idx = 1

                results.append(proba_classes[max_proba_idx])

        return results
             
    def _save_predictions(self, df_predictions, filename):
        predicted_path = os.path.abspath(
            os.path.splitext(filename)[0] + "_predicted.csv")
        df_predictions.to_csv(predicted_path, index=False, encoding='utf-8')
        self.ctx.log('Predictions are saved to %s' % predicted_path)
        return predicted_path



import pandas


class DataFrame(object):
    """Warpper around Pandas DataFrame."""

    def __init__(self):
        super(DataFrame, self).__init__()

    @staticmethod
    def load(filename, target, features=None, nrows=None):
        try:
            df = DataFrame._read_csv(filename, ',', features, nrows)
        except Exception as e:
            df = DataFrame._read_csv(filename, '|', features, nrows)

        features = df.columns.tolist()
        if target in features:
            df.drop(columns=[target], inplace=True)

        return df

    @staticmethod
    def load_records(filename, target, features=None, nrows=None):
        df = DataFrame.load(filename, target, features, nrows)

        features = df.columns.tolist()
        df.replace({pandas.np.nan: None}, inplace=True)
        records = df.values.tolist()

        return records, features

    @staticmethod
    def save(filename, data):
        df = pandas.DataFrame.from_records(data['data'], columns=data['columns'])
        df.to_csv(filename, index=False, encoding='utf-8')

    @staticmethod
    def _read_csv(filename, sep, features=None, nrows=None):
        return pandas.read_csv(filename,
            encoding='utf-8', escapechar="\\", usecols=features,
            na_values=['?'], header=0, prefix=None, sep=sep,
            nrows=nrows, low_memory=False, compression='infer')



import contextlib
import errno
import logging
import os
import os.path
import shutil
import tempfile
import glob
import platform


class LocalFSClient:

    def remove_folder(self, path, remove_self=True):
        shutil.rmtree(path, ignore_errors=True)
        if not remove_self:
            self.create_folder(path)

    def remove_file(self, path, wild=False):
        try:
            if wild:
                for fl in glob.glob(path):
                    os.remove(fl)
            else:
                os.remove(path)
        except OSError:
            pass

    def is_file_exists(self, path):
        return os.path.isfile(path)

    def is_folder_exists(self, path):
        return os.path.exists(path)

    def create_folder(self, path):
        if not self.is_folder_exists(path):
            os.makedirs(path)

        # try:
        # except OSError:
        #     pass
            #logging.exception("createFolder failed")
        self.list_folder(self.get_parent_folder(path))

    def create_parent_folder(self, path):
        parent = os.path.dirname(path)
        self.create_folder(parent)

    def get_parent_folder(self, path):
        return os.path.dirname(path)

    def read_text_file(self, path):
        import codecs

        self.list_folder(self.get_parent_folder(path))
        with codecs.open(path, "r", encoding='utf-8') as file:
            return file.read()

    def write_text_file(self, path, data, atomic=False):
        self.create_parent_folder(path)

        if atomic:
            with self.open_atomic(path, "w") as file:
                file.write(data)
        else:
            from a2ml.api.utils import fsclient
            self.remove_file(path)

            with fsclient.open_file(path, "w") as file:
                try:
                    file.write(data)
                finally:
                    file.flush()  # flush file buffers
                    os.fsync(file.fileno())

        self.read_text_file(path)

    def list_folder(self, path, wild=False, removeFolderName=False, meta_info=False):
        res = []
        try:
            if wild:
                glob_res = glob.glob(path)
                if removeFolderName:
                    len_parent = len(os.path.dirname(path))+1
                    for file in glob_res:
                        res.append(file[len_parent:])
                else:
                    res = glob_res
            else:
                res = os.listdir(path)
        except OSError:
            pass

        if meta_info:
            result_meta = []
            parent_folder = path
            if wild:
                parent_folder = os.path.dirname(path)

            for file_path in res:
                result_meta.append({'path': file_path,
                                    'last_modified': self.getMTime(os.path.join(parent_folder, file_path)),
                                    'size': self.getFileSize(os.path.join(parent_folder, file_path))})

            res = result_meta

        return res

    def get_mtime(self, path):
        return os.path.getmtime(path)

    def get_file_size(self, path):
        return os.path.getsize(path)

    @contextlib.contextmanager
    def open_atomic(self, path, mode):
        parent = self.get_parent_folder(os.path.abspath(path))
        self.list_folder(parent)

        temp_dir = self.get_temp_folder()
        try:
            temp_path = os.path.join(temp_dir, os.path.basename(path))
            with open(temp_path, mode) as f:
                try:
                    yield f
                finally:
                    f.flush()  # flush file buffers
                    os.fsync(f.fileno())  # ensure all data are written to disk
            if platform.system() == "Windows":
                if os.path.exists(path):
                    os.remove(path)

            os.rename(temp_path, path)  # atomic move to target place
        finally:
            self.remove_folder(temp_dir)

    def get_temp_folder(self):
        if os.environ.get('AUGER_LOCAL_TMP_DIR'):
            self.create_folder(os.environ.get('AUGER_LOCAL_TMP_DIR'))

        return tempfile.mkdtemp(dir=os.environ.get('AUGER_LOCAL_TMP_DIR'))

    @contextlib.contextmanager
    def save_atomic(self, path):
        temp_dir = self.get_temp_folder()
        try:
            temp_path = os.path.join(temp_dir, os.path.basename(path))
            yield temp_path
        finally:
            self.remove_folder(temp_dir)

    def move_file(self, path_src, path_dst):
        if platform.system() == "Windows":
            if os.path.exists(path_dst):
                os.remove(path_dst)

        os.rename(path_src, path_dst)  # atomic move to target place

    def copy_file(self, path_src, path_dst):
        if self.is_file_exists(path_dst):
            self.remove_file(path_dst)

        shutil.copy2(path_src, path_dst)

    def copy_files(self, path_src, path_dst):
        if self.is_file_exists(path_dst):
            self.remove_file(path_dst)

        self.create_folder(path_dst)

        for fl in glob.glob(path_src):
            shutil.copy(fl, path_dst)

    def copy_folder(self, path_src, path_dst):
        shutil.copytree(path_src, path_dst)

    def archive_folder(self, path_src, format):
        shutil.make_archive(path_src, format, path_src)

    def download_file(self, path, local_path):
        from urllib.request import urlretrieve
        from auger_ml.Utils import url_encode

        self.create_parent_folder(local_path)
        if local_path and local_path.startswith("/var/src"):
            # For tests
            urlretrieve(url_encode(path), local_path)
        else:
            local_filename, headers = urlretrieve(url_encode(path))
            self.move_file(local_filename, local_path)



from a2ml.api.base_a2ml import BaseA2ML
from a2ml.api.utils.show_result import show_result

class A2MLExperiment(BaseA2ML):
    """Contains the experiment operations that interact with provider."""
    def __init__(self, ctx, provider):
        """Initializes a new a2ml experiment.

        Args:
            context (object): An instance of the a2ml Context.
            provider (str): The automl provider/s you wish to run. For example 'auger,azure,google'.
        
        Returns:
            A2MLExperiment object

        Examples:
            .. code-block:: python

                ctx = Context()
                model = A2MLExperiment(ctx, 'auger, azure')
        """
        super(A2MLExperiment, self).__init__()
        self.ctx = ctx
        self.runner = self.build_runner(ctx, provider, 'experiment')

    @show_result
    def list(self):
        """List all of the experiments for the Project specified in the .yaml.

        Note:
            You will need to user the `iter <https://www.programiz.com/python-programming/methods/built-in/iter>`_ function to access the dataset elements.
        
        Returns:
            Results for each provider. ::

                {
                    'auger': {
                        'result': True,
                        'data': {
                            'experiments': <object> 
                        }
                    }
                }

        Examples:
            .. code-block:: python

                ctx = Context()
                experiment_list = A2MLExperiment(ctx, 'auger, azure').list()
                for provider in ['auger', 'azure']
                    if experiment_list[provider].result is True:
                        for experiment in iter(experiment_list[provider].data.datasets):
                            ctx.log(experiment.get('name'))
                    else:
                        ctx.log('error %s' % experiment_list[provider].data)
        """
        return self.runner.execute('list')

    @show_result
    def start(self):
        """Starts experiment/s for selected dataset. If the name of experiment is not set in context config, new experiment will be created, otherwise an existing experiment will be run.
        
        Returns:
            Results for each provider. ::

                {
                    'auger': {
                        'result': True,
                        'data': {
                            'experiment_name': <experiment_name>,
                            'session_id': <session_id>
                        }
                    }
                }
        
        Examples:
            .. code-block:: python

                ctx = Context()
                experiment = A2MLExperiment(ctx, providers).start()

        """
        return self.runner.execute('start')

    @show_result
    def stop(self):
        """Stops runninng experiment/s.

        Returns:
            Results for each provider. ::

                {
                    'auger': {
                        'result': True,
                        'data': {
                            'stopped': <experiment_name>
                        }
                    }
                }
        
        Examples:
            .. code-block:: python

                ctx = Context()
                experiment = A2MLExperiment(ctx, providers).stop()

        """
        return self.runner.execute('stop')

    @show_result
    def leaderboard(self, run_id):
        """The leaderboard of the currently running or previously completed experiment/s.

        Args:
            run_id (str): The run id for a training session. A unique run id is created for every train. If set to None default is last experiment train.
        
        Returns:
            Results for each provider. ::

                {
                    'auger': {
                        'result': True,
                        'data': {
                            'leaderboard': <experiment_name>
                        }
                    }
                }
        
        Examples:
            .. code-block:: python

                ctx = Context()
                leaderboard = A2MLExperiment(ctx, 'auger, azure').leaderboard()
                for provider in ['auger', 'azure']
                if leaderboard[provider].result is True:
                    for entry in iter(leaderboard[provider].data.leaderboard):
                        ctx.log(entry['model id'])
                        ctx.log('status %s' % leaderboard[provider].data.status)
                else:
                    ctx.log('error %s' % leaderboard[provider].data)

        """
        return self.runner.execute('leaderboard', run_id)

    @show_result
    def history(self):
        """The history of the currently running or previously completed experiment/s.

        Note:
            You will need to user the `iter <https://www.programiz.com/python-programming/methods/built-in/iter>`_ function to access the dataset elements.
        
        Returns:
            Results for each provider. ::

                 {
                    'auger': {
                        'result': True,
                        'data': {
                            'history': <object>
                        }
                    }
                }
        
        Examples:
            .. code-block:: python

                ctx = Context()
                history = A2MLExperiment(ctx, 'auger, azure').history()
                for provider in ['auger', 'azure']
                if history[provider].result is True:
                    for run in iter(history[provider].data.history):
                    ctx.log("run id: {}, status: {}".format(
                        run.get('id'),
                        run.get('status')))
                else:
                    ctx.log('error %s' % history[provider].data)

        """
        return self.runner.execute('history')



import asyncio
import json
import jsonpickle
import requests
import sys
import os

from a2ml.api.a2ml_credentials import A2MLCredentials


def show_output(data):
    if isinstance(data, dict):
        data_type = data.get('type', None)

        if  data_type == 'log':
            sys.stdout.write('\b')
            print(data['msg'])
        elif  data_type == 'result':
            sys.stdout.write('\b')
            print(data['status'] + ':', data['result'])
    elif isinstance(data, Exception):
        sys.stdout.write('\b')
        print('Error:', data)
    else:
        sys.stdout.write('\b')
        print(data)

class RemoteRunner(object):
    CRUD_TO_METHOD = {
        'create': 'post',
        'delete': 'delete',
        'list': 'get',
    }

    NON_CRUD_TO_METHOD = {
        'actual': 'post',
        'deploy': 'patch',
        'evaluate': 'patch',
        'history': 'get',
        'import_data': 'patch',
        'leaderboard': 'get',
        'predict': 'post',
        'review': 'patch',
        'select': 'patch',
        'start': 'patch',
        'stop': 'patch',
        'train': 'patch',
    }

    def __init__(self, ctx, provider, obj_name = None):
        super(RemoteRunner, self).__init__()
        providers = ctx.get_providers(provider)
        if len(providers) == 0:
            raise Exception("Please specify provider")

        provider = providers[0]

        self.ctx = ctx
        self.obj_name = obj_name
        self.ctx.credentials = A2MLCredentials(ctx, provider).load()
        self.server_endpoint = ctx.config.get('server_endpoint', os.environ.get('A2ML_SERVER_ENDPOINT'))
        self.ws_endpoint = 'ws' + self.server_endpoint[4:]

    def _params(self, *args, **kwargs):
        return {
            'context': jsonpickle.encode(self.ctx),
            'project_name': self.ctx.config.get('name'),
            'args': args,
            'kwargs': kwargs,
        }

    def get_http_verb_and_path(self, operation_name):
        crud = False

        if operation_name in self.CRUD_TO_METHOD:
            http_verb = self.CRUD_TO_METHOD[operation_name]
            crud = True
        elif operation_name in self.NON_CRUD_TO_METHOD:
            http_verb = self.NON_CRUD_TO_METHOD[operation_name]
        else:
            raise ValueError(f'unknown operation {operation_name}')

        if self.obj_name:
            if crud:
                path = f'/api/v1/{self.obj_name}s'
            else:
                path = f'/api/v1/{self.obj_name}s/{operation_name}'
        else:
            path = f'/api/v1/{operation_name}'

        return (http_verb, path)

    def execute(self, operation_name, *args, **kwargs):
        http_verb, path = self.get_http_verb_and_path(operation_name)

        res = self.make_requst(http_verb, path, self._params(*args, **kwargs))
        asyncio.get_event_loop().run_until_complete(self.wait_result(res['data']['request_id']))

    def make_requst(self, http_verb, path, params):
        method = getattr(requests, http_verb)
        return self.handle_respone(method(f"{self.server_endpoint}{path}", json=params))

    def handle_respone(self, response):
        if response.status_code == 200:
            return response.json()
        else:
            show_output(f"Request error: {response.status_code} {response.text}")
            raise Exception(f"Request error: {response.status_code} {response.text}")

    def handle_weboscket_respone(self, data):
        data_type = data.get('type', None)

        if data_type == 'result' and isinstance(data['result'], dict):
            config = jsonpickle.decode(data['result']['config'])
            data['result'] = data['result']['response']
            config.write_all()

        show_output(data)

    async def spinning_cursor(self):
        while True:
            for cursor in '|/-\\':
                sys.stdout.write(cursor)
                sys.stdout.flush()
                await asyncio.sleep(0.2)
                sys.stdout.write('\b')

    async def wait_result(self, request_id):
        import websockets

        done = False
        last_msg_id = '0'
        while not done:
            try:
                endpoint = self.ws_endpoint + "/ws?id=" + request_id + '&last_msg_id=' + str(last_msg_id)
                async with websockets.connect(endpoint) as websocket:
                    asyncio.run_coroutine_threadsafe(self.spinning_cursor(), asyncio.get_event_loop())

                    while True:
                        data = await websocket.recv()
                        data = json.loads(data)

                        if '_msg_id' in data:
                            last_msg_id = data['_msg_id']

                        self.handle_weboscket_respone(data)

                        if data.get('type', None) == 'result':
                            done = True
                            break
            except Exception as e:
                show_output(e)
                await asyncio.sleep(2)




from a2ml.api.base_a2ml import BaseA2ML
from a2ml.api.utils.show_result import show_result


class A2ML(BaseA2ML):
    """Facade to A2ML providers."""

    def __init__(self, ctx, provider = None):
        """Initializes A2ML PREDIT instance.

        Args:
            context (object): An instance of the a2ml Context.
            provider (str): The automl provider/s you wish to run. For example 'auger,azure,google'.
        
        Returns:
            A2ML object

        Examples:
            .. code-block:: python

                ctx = Context()
                a2ml = A2ML(ctx, 'auger, azure')
        """
        super(A2ML, self).__init__()
        self.ctx = ctx
        self.runner = self.build_runner(ctx, provider)
        self.local_runner = lambda: self.build_runner(ctx, provider, force_local=True)

    @show_result
    def import_data(self):
        """Imports data defined in context. Uploading the same file name will result in versions being appended to the file name.

        Note:
            Your context points to a config file where ``source`` is defined.

            .. code-block:: yaml

                # Local file name or remote url to the data source file
                source: './dataset.csv'
        
        Returns:
            Results for each provider. ::

                {
                    'auger': {'result': True, 'data': {'created': 'dataset.csv'}}, \n
                    'azure': {'result': True, 'data': {'created': 'dataset.csv'}}
                }
            
            Failures will return error messages. ::

                {
                    'auger': {'result': False, 'data': 'Please specify data source file...'}, \n
                    'azure': {'result': False, 'data': 'Please specify data source file...'}
                }

        
            
        Examples:
            .. code-block:: python

                ctx = Context()
                a2ml = A2ML(ctx, 'auger, azure')
                a2ml.import_data()
        """
        return self.runner.execute('import_data')

    @show_result
    def train(self):
        """Starts training session based on context state.

        Returns:
            Results for each provider. ::

                {
                    'auger': {
                        'result': True,
                        'data': {
                            'eperiment_name': 'dataset.csv-4-experiment',
                            'session_id': '9ccfe04eca67757a'
                         }
                    }
                }

            Failures will return error messages. ::

                {
                    'auger': {'result': False, 'data': 'Please set target to build model.'}
                }

        Examples:

            .. code-block:: python

                ctx = Context()
                a2ml = A2ML(ctx, 'auger, azure')
                a2ml.train()

        """
        return self.runner.execute('train')

    @show_result
    def evaluate(self, run_id = None):
        """Evaluate the results of training.
        
        Args:
            run_id (str, optional): The run id for a training session. A unique run id is created for every train. Default is last experiment train.
            
        Returns:
            Results for each provider. ::

                {
                    'auger': {
                        'result': True,
                        'data': {
                            'run_id': '9ccfe04eca67757a',
                            'leaderboard': [
                                {'model id': 'A017AC8EAD094FD', 'rmse': '0.0000', 'algorithm': 'LGBMRegressor'},
                                {'model id': '4602AFCEEEAE413', 'rmse': '0.0000', 'algorithm': 'ExtraTreesRegressor'}
                            ],
                            'status': 'started'
                        }
                    }
                }

            Status:

                preprocess: Search is preprocessing data for traing
                started: Search is in progress
                completed: Search is completed
                interrupted: Search was interrupted

            Examples:
            .. code-block:: python

                ctx = Context()
                a2ml = A2ML(ctx, 'auger, azure')
                a2ml.evaluate()
        """
        return self.runner.execute('evaluate', run_id = run_id)

    @show_result
    def deploy(self, model_id, locally=False):
        """Deploy a model locally or to specified provider/s.

        Note:
            See evaluate function to get model_id

        Args:
            model_id (str): The model id from any experiment you will deploy
            locally(bool): Deploys the model locally if True, on the Provider Cloud if False. The default is False.
        
        Returns:
            Results for each provider. ::

                {
                    'auger': {
                        'result': True,
                        'data': {'model_id': 'A017AC8EAD094FD'}
                    }
                }

        Examples:
            .. code-block:: python

                ctx = Context()
                a2ml = A2ML(ctx, 'auger, azure')
                a2ml.deploy(model_id='A017AC8EAD094FD')
        """
        return self.__get_runner(locally).execute('deploy', model_id, locally)

    @show_result
    def predict(self, filename, model_id, threshold=None, locally=False):
        """Predict results with new data against deployed model. Predictions are stored next to the file with data to be predicted on. The file name will be appended with suffix _predicted.

        Args:
            filename(str): The file with data to request predictions for.
            model_id(str): The deployed model id you want to use.
            threshold(float, optional): For classification models only. This will return class probabilities with response.
            locally(bool, optional): Predicts using a local model if True, on the Provider Cloud if False. The default is False.
        
        Results:
            Results for each provider. ::

                {
                    'auger': {
                        'result': True,
                        'data': {'predicted': '<path_to_file>/dataset_predicted.csv'}
                        }
                    }
                }

        Examples:
            .. code-block:: python

                ctx = Context()
                a2ml = A2ML(ctx, 'auger, azure')
                a2ml.predict(model_id='D881079E1ED14FB',filename=<path_to_file>/dataset.csv)
           
        """
        return self.__get_runner(locally).execute('predict', filename, model_id, threshold, locally)

    @show_result
    def review(self):
        """Review the performance of deployed model."""
        return self.runner.execute('review')

    def __get_runner(self, locally):
        if locally:
            return self.local_runner()
        else:
            return self.runner



import os
import pytest

from a2ml.cmdl.cmdl import cmdl
from a2ml.api.auger.impl.cloud.rest_api import RestApi


def interceptor(payload, monkeypatch):
    def payloader(x, method, *args, **kwargs):
        return payload[method]
    monkeypatch.setattr(
        RestApi, 'call_ex', payloader)

class TestAugerAuth():
    def test_login(self, log, runner, isolated, monkeypatch):
        PAYLOAD = {
            'create_token': {
                'data': {
                    'token': 'fake_token_for_testing_purpose',
                    },
            },
            'get_organizations': {
                'meta': {
                    'status': 200,
                    'pagination':
                        {'limit': 100, 'total': 1, 'count': 1, 'offset': 0}
                },
                'data': [{'name': 'auger'}]
            }
        }
        interceptor(PAYLOAD, monkeypatch)
        monkeypatch.setenv("AUGER_CREDENTIALS_PATH", os.getcwd())
        result = runner.invoke(
            cmdl,
            ['auth', 'login'],
            input="test@example.com\nauger\npassword\n")
        assert result.exit_code == 0
        assert (log.records[-1].message ==
                "[config] You are now logged in on https://app.auger.ai"
                " as test@example.com.")

    def test_logout(self, log, runner, isolated, monkeypatch, auger_authenticated):
        result = runner.invoke(cmdl, ['auth', 'logout'])
        assert result.exit_code == 0
        assert log.records[-1].message == "[config] You are logged out of Auger."

    def test_whoami_anonymous(self, log, runner, monkeypatch):
        monkeypatch.setenv("AUGER_CREDENTIALS", '{}')
        result = runner.invoke(cmdl, ['auth', 'whoami'])
        #assert result.exit_code != 0
        assert (log.records[-1].message ==
                "[config] Please login to Auger...")

    def test_whoami_authenticated(self, log, runner, monkeypatch, auger_authenticated):
        result = runner.invoke(cmdl, ['auth', 'whoami'])
        assert result.exit_code == 0
        assert (log.records[-1].message ==
                "[config] test_user auger https://example.com")

    def test_logout_not_logged(self, log, runner, isolated, monkeypatch):
        monkeypatch.setenv("AUGER_CREDENTIALS", '{}')
        result = runner.invoke(cmdl, ['auth', 'logout'])
        assert (log.records[-1].message == '[config] You are not logged in Auger.')
        #assert result.exit_code != 0



import os

from a2ml.cmdl.cmdl import cmdl
from a2ml.api.utils.context import Context

class TestNewCommand():

    def test_minimal_arguments_successfull_creation(self, runner, isolated):
        # successful status
        result = runner.invoke(cmdl, ['new', 'test_project'])
        assert result.exit_code == 0

        # directory created
        target_dir = os.path.join(os.getcwd(), 'test_project')
        assert os.path.exists(target_dir) and os.path.isdir(target_dir)

        # config file exists
        config_file = os.path.join(target_dir, 'auger.yaml')
        assert os.path.exists(config_file)

        # config contains proper data
        config = Context(path=target_dir).config
        assert config.get('name', '') == 'test_project'

    def test_project_with_given_name_already_exists(self, runner, log, isolated):
        os.chdir('..')
        runner.invoke(cmdl, ['new', 'test_project'])
        result = runner.invoke(cmdl, ['new', 'test_project'])
        #assert result.exit_code != 0
        assert (log.records[-1].message ==
                "[config] Can't create 'test_project'. Folder already exists.")

    def test_nested_project_forbidden(self, runner, log, isolated):
        result = runner.invoke(cmdl, ['new', 'test_project'])
        #assert result.exit_code != 0
        assert (log.records[-1].message ==
                "[config] To build your model, please do:"
                " cd test_project && a2ml import && a2ml train")

    def test_full_set_of_arguments(self, log, runner, isolated, project):
        os.chdir('..')
        result = runner.invoke(
            cmdl, [
                'new', 'new_project',
                '--model-type', 'regression',
                '--target', 'target_column',
                '--source', 'cli-integration-test/iris.csv'])

        assert result.exit_code == 0
        
        target_dir = os.path.join(os.getcwd(), 'new_project')
        config = Context(path=target_dir).config

        assert config.get('model_type', '') == 'regression'
        assert config.get('target', '') == 'target_column'
        assert config.get('source', '') == os.path.join(
            os.getcwd(), 'cli-integration-test', 'iris.csv')

    def test_bad_source(self, log, runner, isolated):
        result = runner.invoke(
            cmdl, ['new', 'test_project', '--source', 'not_existing_file.csv'])
        #assert result.exit_code != 0
        assert log.messages[-1].startswith("[config] Can't find file to import:")

    def test_source_wrong_extension(self, log, runner, isolated):
        result = runner.invoke(
            cmdl, ['new', 'test_project', '--source', 'file_with_wrong.extension'])
        #assert result.exit_code != 0
        assert log.messages[-1] ==\
             '[config] Source file has to be one of the supported fomats: .csv, .arff, .gz, .bz2, .zip, .xz, .json, .xls, .xlsx, .feather, .h5, .hdf5'
